from matplotlib import pyplot as plt 
import pandas as pd
import numpy as np
from scipy import signal
from scipy import integrate
import math

#%%
#imports data
vib_data = pd.read_csv('C:/Users/Frode/Desktop/kalibrering/konfig_spm_2404_2g_1mill_mm_1.csv')
#%%
#set constants
RPM=    1300# axel rounds per minute
g=      9.81#tyngdeakselerasjon i m/s2
Bntot=  7#total number of averageing blocks
Bn=     6# number of averageing blocks used
Nq=     2.56# nyquist theoreme constant
Wc=     2#windowing amplitude correction
Ar=     33.7#Axel rotations per block;10 is minimum rotations of axel according to ISO20816-5
Range=  2.048#ADXL355 Mesurment range in g
ADXL355_res=2**20# Adxl355 20 bit resolution
Lsb_g=int(ADXL355_res/(2*Range))#Least significant bits per g
Ttot=vib_data.Time.iat[-1]#Total messuerment time
Ntot=pd.DataFrame.last_valid_index(vib_data)#Number of total samples
Fs=round(Ntot/Ttot,2) #sample rate
Bt=60/RPM*Ar#Block time
Bl=int(Fs*Bt)#Block Length
Fmax=int(Fs/Nq)#Max frequency on FFT
Lor=int(Bl//Nq)#Lines of resolution on FFT
#necessary Ttot > Bt*Bntot
if Ttot<Bt*Bntot:
    raise ValueError("Too Short Mesurment Time")
print ("LOR=", Lor)
print ("FS",Fs)
print ("oppløsning",(Fs/Bl))
print ("fmax",Fmax)
#%%
#Puts data in array and convert "Raw" data to millimeter/second squarde(mm/s2) acceleration 
time_array = np.array(vib_data.Time)
x_axis_array = np.array(vib_data.Accel_X_RAW)/Lsb_g*(g*1000)
y_axis_array = np.array(vib_data.Accel_Y_RAW)/Lsb_g*(g*1000)
z_axis_array = np.array(vib_data.Accel_Z_RAW)/Lsb_g*(g*1000)

#Convert 1-d arrays into 6blocks matrix, remove first block from total signal
time_array = time_array[len(time_array)-(Bl*Bn):]
x_axis_array=x_axis_array[len(x_axis_array)-(Bl*Bn):]
y_axis_array=y_axis_array[len(y_axis_array)-(Bl*Bn):]
z_axis_array=z_axis_array[len(z_axis_array)-(Bl*Bn):] #cuts away first readings

time_matrix=time_array.reshape(Bn,Bl)
x_axis_matrix=x_axis_array.reshape(Bn,Bl)# creates BnxBl matrix
y_axis_matrix=y_axis_array.reshape(Bn,Bl)
z_axis_matrix=z_axis_array.reshape(Bn,Bl)

#Remove DC, static accelration from signal
x_axis_matrix=signal.detrend(x_axis_matrix,axis=1)
y_axis_matrix=signal.detrend(y_axis_matrix,axis=1)
z_axis_matrix=signal.detrend(z_axis_matrix,axis=1)

#plt.plot(time_matrix[[0], : ].reshape(Bl,),z_axis_matrix[[0], : ].reshape(Bl,))

#Calculate RMS accel accordig to ISO 20816-5
x_RMS_block=np.sqrt(np.sum(np.square(x_axis_matrix),axis=1)/Bl)
x_axis_RMS=x_RMS_block.mean(axis=0)

y_RMS_block=np.sqrt(np.sum(np.square(y_axis_matrix),axis=1)/Bl)
y_axis_RMS=y_RMS_block.mean(axis=0)

z_RMS_block=np.sqrt(np.sum(np.square(z_axis_matrix),axis=1)/Bl)
z_axis_RMS=z_RMS_block.mean(axis=0)

Absolute_vib_RMS=math.sqrt(x_axis_RMS**2+y_axis_RMS**2+z_axis_RMS**2)

#Convert acceleration matrix to velocity matrix using cummulative trapez integration.
velocity_x_matrix=integrate.cumtrapz(x_axis_matrix,time_matrix,axis=1,initial=0)
velocity_y_matrix=integrate.cumtrapz(y_axis_matrix,time_matrix,axis=1,initial=0)
velocity_z_matrix=integrate.cumtrapz(z_axis_matrix,dx=1/Fs,axis=1,initial=0)

#visualisation of velocity signal on z axis.
#plt.plot(time_matrix[[0], : ].reshape(Bl,),velocity_z_matrix[[0], : ].reshape(Bl,))
#plt.plot(time_matrix[[1], : ].reshape(Bl,),velocity_z_matrix[[1], : ].reshape(Bl,))
#plt.plot(time_matrix[[2], : ].reshape(Bl,),velocity_z_matrix[[2], : ].reshape(Bl,))
#plt.plot(time_matrix[[3], : ].reshape(Bl,),velocity_z_matrix[[3], : ].reshape(Bl,))
#plt.plot(time_matrix[[4], : ].reshape(Bl,),velocity_z_matrix[[4], : ].reshape(Bl,))
#plt.plot(time_matrix[[5], : ].reshape(Bl,),velocity_z_matrix[[5], : ].reshape(Bl,))

#Calculate RMS velocity according to ISO 20816-5
RMSv_x_block=np.sqrt(np.sum(np.square(velocity_x_matrix),axis=1)/Bl)
x_axis_RMSv=RMSv_x_block.mean(axis=0)

RMSv_y_block=np.sqrt(np.sum(np.square(velocity_y_matrix),axis=1)/Bl)
y_axis_RMSv=RMSv_y_block.mean(axis=0)

RMSv_z_block=np.sqrt(np.sum(np.square(velocity_z_matrix),axis=1)/Bl)
z_axis_RMSv=RMSv_z_block.mean(axis=0)


Absolute_vib_RMS_velocity=math.sqrt(x_axis_RMSv**2+y_axis_RMSv**2+z_axis_RMSv**2)
print("RMS v=",round(z_axis_RMSv,2),"mm/s")
print("RMS a=",round(z_axis_RMS/1000,2),"m/s2")
#%%
#Apply hanning window to signal
hanning=np.hanning(Bl)

vel_x_han=velocity_x_matrix*hanning
vel_y_han=velocity_y_matrix*hanning
vel_z_han=velocity_z_matrix*hanning

acc_x_han=x_axis_matrix*hanning
acc_y_han=y_axis_matrix*hanning
acc_z_han=z_axis_matrix*hanning
#%%
#FFT velocity whit normalization and multiplied by Windowing amplitude correction factor.
vel_x_fft=np.fft.fft(vel_x_han,axis=1,)*Wc*(2/Bl) 
vel_y_fft=np.fft.fft(vel_y_han,axis=1,)*Wc*(2/Bl)
vel_z_fft=np.fft.fft(vel_z_han,axis=1,)*Wc*(2/Bl)

#combining the 6 blocks to one signal
vel_x_fft_mean=vel_x_fft.mean(axis=0)
vel_y_fft_mean=vel_y_fft.mean(axis=0)
vel_z_fft_mean=vel_z_fft.mean(axis=0)

x=np.linspace(0,Fmax,num=Lor) #Set plot's x-axis lenght an resolution
yx=np.abs(vel_x_fft_mean[0:Lor]) #Absolut fft from zero to LOR
plt.figure("fft x akse")
plt.plot(x,yx)
plt.xlabel("Frequency Hz")
plt.ylabel("Velocity pk mm/s")

yy=np.abs(vel_y_fft_mean[0:Lor])#Absolut fft from zero to LOR
plt.figure("fft y akse")
plt.plot(x,yy)
plt.xlabel("Frequency Hz")
plt.ylabel("Velocity pk mm/s")

yz=np.abs(vel_z_fft_mean[0:Lor])#Absolut fft from zero to LOR
plt.figure("fft z akse")
plt.plot(x,yz)
plt.xlabel("Frequency Hz")
plt.ylabel("Velocity pk mm/s")
#%%
#FFT acceleration whit normalization and multiplied by Windowing amplitude correction factor.
acc_x_fft=np.fft.fft(acc_x_han,axis=1,)*Wc*(2/Bl) 
acc_y_fft=np.fft.fft(acc_y_han,axis=1,)*Wc*(2/Bl)
acc_z_fft=np.fft.fft(acc_z_han,axis=1,)*Wc*(2/Bl)

#combining the 6 blocks to one signal
acc_x_fft_mean=acc_x_fft.mean(axis=0)
acc_y_fft_mean=acc_y_fft.mean(axis=0)
acc_z_fft_mean=acc_z_fft.mean(axis=0)

x=np.linspace(0,Fmax,num=Lor)#Set plot's x-axis lenght an resolution
yx=np.abs(acc_x_fft_mean[0:Lor])#Absolut fft from zero to LOR
plt.figure("fft acc x akse")
plt.plot(x,yx)
plt.xlabel("Frequency Hz")
plt.ylabel("Acceleration pk mm/s2")

yy=np.abs(acc_y_fft_mean[0:Lor])#Absolut fft from zero to LOR
plt.figure("fft acc y akse")
plt.plot(x,yy)
plt.xlabel("Frequency Hz")
plt.ylabel("Acceleration pk mm/s2")

yz=np.abs(acc_z_fft_mean[0:Lor])#Absolut fft from zero to LOR
plt.figure("fft acc z akse")
plt.plot(x,yz)
plt.xlabel("Frequency Hz")
plt.ylabel("Acceleration pk mm/s2")
#%%
#Parsevals theorem, energy should be the same in time and frequency domain
x1=z_axis_matrix
T1=1/Fs
yf1=acc_z_fft_mean
N1=len(x1)
ts_energy1 = sum(np.power(x1,2))*T1 # multiply by dt=T1
ts_energy1=ts_energy1.mean(axis=0)
fr_energy1 = sum(np.power(np.abs(yf1),2))*N1*T1 # multiply by df=N1*T1
#%%
print("Time signal energy",ts_energy1)
print ("Frequencydomain energy",fr_energy1)